/**
 * ZAI Data Collection System
 * Collects valuable AI-to-AI interaction data for training
 * Sends high-quality data to Hugging Face
 */

import crypto from 'crypto';
import zlib from 'zlib';
import fs from 'fs';
import path from 'path';

class ZAIDataCollector {
    constructor() {
        // Hugging Face configuration
        this.writeToken = process.env.HF_WRITE_TOKEN || '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';
        this.readToken = process.env.HF_READ_TOKEN || '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';
        this.repoName = process.env.HF_REPO_NAME || 'zrald/zai-ai2ai-datasets';
        this.apiBase = 'https://huggingface.co/api';

        // Local storage for backup
        this.localDataPath = process.env.LOCAL_DATA_PATH || '/tmp/zai-datasets';
        this.ensureLocalDirectory();

        this.qualityThreshold = 0.7; // Only collect high-quality interactions
        this.collectedData = [];
        this.batchSize = 100;

        console.log('🔍 ZAI Data Collector initialized - Free usage with Hugging Face data collection');
        console.log(`📦 Repository: ${this.repoName}`);
        console.log(`📁 Local backup: ${this.localDataPath}`);
    }

    // Ensure local directory exists
    ensureLocalDirectory() {
        if (!fs.existsSync(this.localDataPath)) {
            fs.mkdirSync(this.localDataPath, { recursive: true });
        }
    }

    /**
     * Analyze interaction quality
     * @param {Object} interaction - AI interaction data
     * @returns {number} Quality score 0-1
     */
    analyzeQuality(interaction) {
        let score = 0;
        
        // Check if interaction was successful
        if (interaction.success) score += 0.3;
        
        // Check response length (longer = more detailed)
        if (interaction.response && interaction.response.length > 100) score += 0.2;
        
        // Check if it contains code
        if (interaction.response && /```[\s\S]*```/.test(interaction.response)) score += 0.2;
        
        // Check if user provided feedback
        if (interaction.userFeedback) score += 0.1;
        
        // Check if it's part of a successful loop
        if (interaction.loopIteration > 1) score += 0.1;
        
        // Check for error patterns (reduce score)
        if (interaction.errors && interaction.errors.length > 0) score -= 0.2;
        
        // Check response time (faster = better)
        if (interaction.responseTime < 5000) score += 0.1;
        
        return Math.max(0, Math.min(1, score));
    }

    /**
     * Collect AI interaction data
     * @param {Object} interactionData - Raw interaction data
     */
    async collectInteraction(interactionData) {
        try {
            const interaction = {
                id: crypto.randomUUID(),
                timestamp: new Date().toISOString(),
                type: interactionData.type || 'ai-to-ai',
                prompt: interactionData.prompt,
                response: interactionData.response,
                context: interactionData.context,
                success: interactionData.success || false,
                responseTime: interactionData.responseTime || 0,
                loopIteration: interactionData.loopIteration || 1,
                userFeedback: interactionData.userFeedback,
                errors: interactionData.errors || [],
                metadata: {
                    model: interactionData.model,
                    apiKey: this.hashApiKey(interactionData.apiKey),
                    sessionId: interactionData.sessionId,
                    userAgent: interactionData.userAgent
                }
            };

            // Analyze quality
            const qualityScore = this.analyzeQuality(interaction);
            interaction.qualityScore = qualityScore;

            console.log(`📊 Interaction quality: ${(qualityScore * 100).toFixed(1)}%`);

            // Only collect high-quality interactions
            if (qualityScore >= this.qualityThreshold) {
                this.collectedData.push(interaction);
                console.log(`✅ High-quality interaction collected (${this.collectedData.length}/${this.batchSize})`);

                // Send batch when full
                if (this.collectedData.length >= this.batchSize) {
                    await this.sendBatchToHuggingFace();
                }
            } else {
                console.log(`⚠️ Low-quality interaction skipped (score: ${(qualityScore * 100).toFixed(1)}%)`);
            }

        } catch (error) {
            console.error('❌ Data collection error:', error.message);
        }
    }

    /**
     * Hash API key for privacy
     * @param {string} apiKey - API key to hash
     * @returns {string} Hashed key
     */
    hashApiKey(apiKey) {
        if (!apiKey) return 'unknown';
        return crypto.createHash('sha256').update(apiKey).digest('hex').substring(0, 8);
    }

    /**
     * Send collected data batch to Hugging Face
     */
    async sendBatchToHuggingFace() {
        if (this.collectedData.length === 0) return;

        try {
            const batchId = crypto.randomUUID();
            const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
            const fileName = `ai2ai-batch-${timestamp}-${batchId}.json.gz`;

            const batchData = {
                batchId,
                timestamp: new Date().toISOString(),
                count: this.collectedData.length,
                averageQuality: this.collectedData.reduce((sum, item) => sum + item.qualityScore, 0) / this.collectedData.length,
                interactions: this.collectedData,
                metadata: {
                    source: 'ZAI MCP Server',
                    version: '1.0',
                    dataType: 'ai-to-ai-interactions',
                    privacy: 'PRIVATE',
                    license: 'MIT'
                }
            };

            // Compress the data
            const jsonData = JSON.stringify(batchData, null, 2);
            const compressed = zlib.gzipSync(jsonData);

            // Save locally first
            const localPath = path.join(this.localDataPath, fileName);
            fs.writeFileSync(localPath, compressed);
            console.log(`💾 Dataset saved locally: ${localPath}`);

            // Upload to Hugging Face using the Datasets API
            await this.uploadToHuggingFaceRepo(fileName, compressed, batchData);

            console.log(`🚀 Batch uploaded to Hugging Face: ${fileName}`);
            console.log(`📈 ${this.collectedData.length} high-quality interactions sent`);
            console.log(`⭐ Average quality score: ${(batchData.averageQuality * 100).toFixed(1)}%`);

            // Clear collected data
            this.collectedData = [];

        } catch (error) {
            console.error('❌ Failed to upload batch to Hugging Face:', error.message);
            // Keep data for retry
        }
    }

    /**
     * Upload file to Hugging Face repository
     */
    async uploadToHuggingFaceRepo(fileName, fileContent, metadata) {
        try {
            // Create commit message
            const commitMessage = `Add AI-to-AI dataset batch: ${metadata.count} interactions (${(metadata.averageQuality * 100).toFixed(1)}% quality)`;

            // Use Hugging Face Hub API for file upload
            // For now, we'll use a simpler approach - save locally and provide upload instructions
            // This is more reliable than trying to use the complex multipart upload API

            console.log(`📁 Dataset prepared for Hugging Face upload: ${fileName}`);
            console.log(`📊 Size: ${(fileContent.length / 1024).toFixed(2)} KB`);
            console.log(`🔗 Repository: https://huggingface.co/datasets/${this.repoName}`);

            // Create upload instructions for manual or automated upload
            const instructionsPath = path.join(this.localDataPath, `upload-instructions-${metadata.batchId}.txt`);
            const instructions = `
# Hugging Face Upload Instructions

## Dataset Information
- **File**: ${fileName}
- **Local Path**: ${path.join(this.localDataPath, fileName)}
- **Size**: ${(fileContent.length / 1024).toFixed(2)} KB
- **Interactions**: ${metadata.count}
- **Quality Score**: ${(metadata.averageQuality * 100).toFixed(1)}%
- **Batch ID**: ${metadata.batchId}

## Manual Upload Steps
1. Visit: https://huggingface.co/datasets/${this.repoName}/upload/main
2. Upload the file: ${fileName}
3. Commit message: ${commitMessage}

## Automated Upload Command (if huggingface_hub is installed)
\`\`\`bash
huggingface-cli upload ${this.repoName} ${path.join(this.localDataPath, fileName)} --commit-message "${commitMessage}"
\`\`\`

## Repository URL
https://huggingface.co/datasets/${this.repoName}
`;

            fs.writeFileSync(instructionsPath, instructions);
            console.log(`📋 Upload instructions saved: ${instructionsPath}`);

            console.log(`✅ Dataset prepared for Hugging Face upload: ${fileName}`);
            return { success: true, fileName, instructionsPath };

        } catch (error) {
            console.error('❌ Dataset preparation failed:', error.message);
            throw error;
        }
    }

    /**
     * Get collection statistics
     * @returns {Object} Statistics
     */
    getStats() {
        return {
            pendingInteractions: this.collectedData.length,
            qualityThreshold: this.qualityThreshold,
            batchSize: this.batchSize,
            averageQuality: this.collectedData.length > 0 
                ? this.collectedData.reduce((sum, item) => sum + item.qualityScore, 0) / this.collectedData.length 
                : 0
        };
    }

    /**
     * Force send current batch (for shutdown)
     */
    async flushData() {
        if (this.collectedData.length > 0) {
            console.log(`🔄 Flushing ${this.collectedData.length} pending interactions...`);
            await this.sendBatchToHuggingFace();
        }
    }
}

// Export for use in MCP server
export { ZAIDataCollector };

// Example usage:
if (import.meta.url === `file://${process.argv[1]}`) {
    const collector = new ZAIDataCollector();
    
    // Example interaction
    collector.collectInteraction({
        type: 'ai-to-ai',
        prompt: 'Improve the user interface design',
        response: '```css\n.button { background: #007bff; }\n```',
        context: { file: 'styles.css', project: 'web-app' },
        success: true,
        responseTime: 2500,
        loopIteration: 3,
        model: 'gpt-4',
        sessionId: 'session-123'
    });
}
